#!/usr/bin/python3
# ==============================================================================================
import tornado.httpserver
import tornado.ioloop
import tornado.web
import tornado.websocket
import tornado.gen
from tornado.options import define, options
import os
import time
import multiprocessing
import json
import serialprocess
# ==============================================================================================
define("port", default=8080, help="run on the given port", type=int)
clients = [] 
input_queue = multiprocessing.Queue()
output_queue = multiprocessing.Queue()
# ==============================================================================================
class IndexHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('index.html')
# ---------------------------------------------------------------------------------------------- 
class StaticFileHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('interface.js')
# ---------------------------------------------------------------------------------------------- 
class WebSocketHandler(tornado.websocket.WebSocketHandler):
    def open(self):
        print('new connection')
        clients.append(self)
        self.write_message("[I]0.0.2")
    # ------------------------------------------------------------------------------------------ 
    def on_message(self, message):
        print('tornado received from client: %s' % json.dumps(message))
        cmd=message[:3]
        if   (cmd == "[D]"):
            input_queue.put(message[3:])
        elif (cmd == "[T]"):
            sp.table=int(message[3])
        else:
            self.write_message("unsupported command")
    # ------------------------------------------------------------------------------------------ 
    def on_close(self):
        print('connection closed')
        clients.remove(self)
# ==============================================================================================
## check the queue for pending messages, and rely that to all connected clients
def checkQueue():
    if not output_queue.empty():
        message = output_queue.get()
        for c in clients:
            c.write_message("[D]"+message)
# ==============================================================================================
if __name__ == '__main__':
    # -----------------------------------------------------------------------------------------
    # Get web path from start line
    # -----------------------------------------------------------------------------------------
    import sys,os
    pth=os.path.dirname(sys.argv[0])
    print("INFO:  PATH " + pth)
    # -----------------------------------------------------------------------------------------
    ## start the serial worker in background (as a daemon)
    # -----------------------------------------------------------------------------------------
    sp = serialprocess.SerialProcess(input_queue, output_queue)
    sp.daemon = True
    sp.start()
    tornado.options.parse_command_line()
    app = tornado.web.Application(
        handlers=[
            (r"/", IndexHandler),
            (r"/static/(.*)", tornado.web.StaticFileHandler, {'path': pth}),
            (r"/ws", WebSocketHandler)
        ]
    )
    httpServer = tornado.httpserver.HTTPServer(app)
#   httpServer.listen(options.port,address='192.168.1.250')
    httpServer.listen(options.port)
    print("Listening on port:", options.port)

    mainLoop = tornado.ioloop.IOLoop.instance()
    ## adjust the scheduler_interval according to the frames sent by the serial port
    scheduler_interval = 100
    scheduler = tornado.ioloop.PeriodicCallback(checkQueue, scheduler_interval, io_loop = mainLoop)
    scheduler.start()
    mainLoop.start()
# ==============================================================================================
